/*
 * (c) Copyright 1992 by Panagiotis Tsirigotis
 * (c) Portions Copyright 1998-2001 by Rob Braun
 * All rights reserved.  The file named COPYRIGHT specifies the terms 
 * and conditions for redistribution.
 */

#include "config.h"
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "str.h"

/*
 * MISCELLANEOUS FUNCTIONS
 */

#include "misc.h"


/*
 * Create a new argv array,
 * copy the original to the new one,
 * and clear the old one
 */
char **argv_copy_and_clear( char **org_argv, int start, int count )
{
   char **new_argv ;
   char *p ;
   int i ;
   int j ;

   new_argv = (char **) malloc( count * sizeof( char * ) ) ;
   if ( new_argv == NULL )
      return( NULL ) ;

   for ( i = 0 ; i < count ; i++ )
   {
      new_argv[ i ] = new_string( org_argv[ start+i ] ) ;
      if ( new_argv[ i ] == NULL )
      {
         for ( j = i-1 ; j >= 0 ; j-- )
            free( new_argv[ j ] ) ;
         free( (char *) new_argv ) ;
         return( NULL ) ;
      }
      for ( p = org_argv[ start+i ] ; *p ; p++ )
         *p = ' ' ;
   }
   return( new_argv ) ;
}


/*
 * We always return a pointer in pathname
 */
#if !defined(linux)
char *basename( char *pathname )
{
   char *s = strrchr( pathname, '/' ) ;

   if ( s == NULL )
      return( pathname ) ;
   else
      return( s+1 ) ;
}
#endif


/*
 * We always return a malloced string
 *
 * There are 2 special cases:
 *
 *      1) pathname == "/"
 *            In this case we return "/"
 *      2) pathname does not contain a '/'
 *            In this case we return "."
 */
char *dirname( const char *pathname )
{
   int len ;
   char *s = strrchr( pathname, '/' ) ;
   char *p ;

   if ( s == NULL )
      return( new_string( "." ) ) ;
   else
   {
      len = s - pathname ;
      if ( len == 0 )
         return( new_string( "/" ) ) ;
   }

   p = (char *)malloc( len+1 ) ;
   if ( p == NULL )
      return( NULL ) ;
   else
   {
      strncpy( p, pathname, len )[ len ] = '\0' ;
      return( p ) ;
   }
}


char *make_pathname( register unsigned count, ... )
{
   va_list ap ;
   register unsigned i ;
   register unsigned len = 0 ;
   register char *s, *p ;
   char *pathname ;

   if ( count == 0 )
      return( NULL ) ;

   va_start( ap, count ) ;
   for ( i = 0 ; i < count ; i++ )
   {
      s = va_arg( ap, char * ) ;
      len += strlen( s ) ;
   }
   va_end( ap ) ;

   pathname = (char *)malloc( len + count ) ;
   if ( pathname == NULL )
      return( NULL ) ;

   p = pathname ;
   va_start( ap, count ) ;
   for ( i = 0 ; i < count ; i++ )
   {
      s = va_arg( ap, char * ) ;
      while ( (*p++ = *s++) ) ;
      *(p-1) = '/' ;         /* change '\0' to '/' */
   }
   *(p-1) = '\0' ;
   va_end( ap ) ;
   return( pathname ) ;
}


