/*
 * (c) Copyright 1992 by Panagiotis Tsirigotis
 * (c) Sections Copyright 1998-2001 by Rob Braun
 * All rights reserved.  The file named COPYRIGHT specifies the terms 
 * and conditions for redistribution.
 */

#include "config.h"
#include "flags.h"
#include "defs.h"
#include "mask.h"
#include "main.h"
#include "reconfig.h"
#include "retry.h"
#include "internals.h"
#include "child.h"
#include "access.h"

#include "msg.h"

/*
 * Control flags
 *
 *      NOENV means that the setjmp buffer should be disabled while the
 *      flag action is running (so if something goes wrong in the flag
 *      action the program will be terminated).
 */
#define CF_NONE                  0x0
#define CF_NOENV                 0x1

struct flag_action
{
   int          fa_value ;
   voidfunc     fa_action ;
   int          fa_control_flags ;
} ;


static struct flag_action flag_actions[] =
{
   { CHILD_FLAG,              child_exit,             CF_NONE      },
   { CPS_RESTART_FLAG,        cps_service_restart,    CF_NONE      },
   { HARD_RECONFIG_FLAG,      hard_reconfig,          CF_NOENV     },
   { RETRY_FLAG,              server_retry,           CF_NONE      },
   { DUMP_FLAG,               dump_internal_state,    CF_NONE      },
   { TERMINATE_FLAG,          terminate_program,      CF_NONE      },
   { CONSISTENCY_FLAG,        user_requested_check,   CF_NONE      },
   { QUIT_FLAG,               quit_program,           CF_NONE      },
   { PERIODIC_CHECK_FLAG,     periodic_check,         CF_NONE      },
   { 0,                       0,                      0            }
} ;


/*
 * Returns after all flags are clear
 */
void check_flags(void)
{
   bool_int valid_env = ps.rws.env_is_valid ;

   while ( ! M_ARE_ALL_CLEAR( ps.flags ) )
   {
      struct flag_action *fap ;

      for ( fap = flag_actions ; fap->fa_value != 0 ; fap++ )
         if ( M_IS_SET( ps.flags, fap->fa_value ) )
         {
            M_CLEAR( ps.flags, fap->fa_value ) ;

            if ( fap->fa_control_flags & CF_NOENV )
               ps.rws.env_is_valid = FALSE ;
            else
               ps.rws.env_is_valid = valid_env ;
            if ( debug.on )
               msg( LOG_DEBUG, "check_flags", "flag entry %d processed", (fap-flag_actions)/sizeof(struct flag_action) ) ;

            (*fap->fa_action)() ;

            break ;
         }
   }

   ps.rws.env_is_valid = valid_env ;
}

